{****************************************************}
{ 		CArtClassDoc.p}
{}
{		The ArtClassDoc Class}
{}
{		SUPERCLASS = CDocument}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CArtClassDoc;

interface

	uses
		Printing, TCL, MoreTCL, ArtClassIntf;


{** Class Constants **}

	const
		WINDmyDoc = 1000;		{ Resource ID for document	windows	}
		DLOG_PRINT = 2000;	{ Dialog to display when printing		}
		strABORTPRINT = 8;		{ String index in STRcommon			}

implementation


{****************************************************}
{ IArtClassDoc}
{}
{		Initialize an ArtClassDoc object}
{}
{****************************************************}

	procedure CArtClassDoc.IArtClassDoc (aSupervisor: CApplication);
	begin
		IDocument(aSupervisor, TRUE);
	end;


{****************************************************}
{ DoCommand (OVERRIDE)}
{}
{		Execute a command}
{}
{****************************************************}

	procedure CArtClassDoc.DoCommand (theCommand: longint);
	begin
		case theCommand of

			cmdOne: 
				;			{ Debugging commands				}
					{ Add code here to experiment		}

			cmdTwo: 
				;

			cmdThree: 
				;

			otherwise
				inherited DoCommand(theCommand);
		end;
	end;


{****************************************************}
{ UpdateMenus (OVERRIDE)}
{}
{		Perform menu management tasks}
{}
{****************************************************}

	procedure CArtClassDoc.UpdateMenus;
		var
			macMenu: MenuHandle;

	begin
		inherited UpdateMenus;

		if gGopher <> itsMainPane then
			Exit(UpdateMenus);

		if gFontItem > 0 then
			begin
				macMenu := GetMHandle(MENUfont);
				if macMenu <> nil then
					CheckItem(macMenu, gFontItem, TRUE);
			end;

		if gSizeItem > 0 then
			begin
				macMenu := GetMHandle(MENUsize);
				if macMenu <> nil then
					CheckItem(macMenu, gSizeItem, TRUE);
			end;

		if gFontStyle = [] then
			gBartender.CheckMarkCmd(cmdPlain, TRUE)
		else
			begin
				if bold in gFontStyle then
					gBartender.CheckMarkCmd(cmdBold, TRUE);

				if italic in gFontStyle then
					gBartender.CheckMarkCmd(cmdItalic, TRUE);

				if underline in gFontStyle then
					gBartender.CheckMarkCmd(cmdUnderline, TRUE);

				if outline in gFontStyle then
					gBartender.CheckMarkCmd(cmdOutline, TRUE);

				if shadow in gFontStyle then
					gBartender.CheckMarkCmd(cmdShadow, TRUE);

				if condense in gFontStyle then
					gBartender.CheckMarkCmd(cmdCondense, TRUE);

				if extend in gFontStyle then
					gBartender.CheckMarkCmd(cmdExtend, TRUE);

			end;

		gBartender.CheckMarkCmd(gLineSpacing, TRUE);
	end;


{****************************************************}
{ NewFile (OVERRIDE)}
{}
{		Set up a document with a new file, usually in response the "New"}
{		command in the File menu.}
{}
{****************************************************}

	procedure CArtClassDoc.NewFile;
		var
			wTitle: Str255;
			wNumber: Str255;
			wCount: integer;

	begin
		if BuildWindow(nil) then
			begin							{ Build a new window					}

				itsWindow.GetTitle(wTitle);	{ Append an index number to the	}
										{   default name of the window		}

				wCount := gDecorator.GetWCount;
				NumToString(wCount, wNumber);
				wTitle := concat(wTitle, '-');
				wTitle := concat(wTitle, wNumber);
				itsWindow.SetTitle(wTitle);

				itsWindow.Select;			{ Make it the active window				}

			end
		else
			begin							{ Not enough memory to make a new	}
										{   Document								}
				gError.PostAlert(STRmemWarn, 2);
				Free;
			end;
	end;


{****************************************************}
{ OpenFile (OVERRIDE)}
{}
{		Open a file for a document, usually in response the "Open"}
{		command in the File menu.}
{}
{****************************************************}

	procedure CArtClassDoc.OpenFile (macSFReply: SFReply);
		var
			theFile: CPNTGFile;
			theBitMap: CBitMap;
			macErr: OSErr;

	begin
		new(theFile);						{ Make a File object and read the	}
		theFile.IPNTGFile;				{   data into a new BitMap			}
		theFile.SFSpecify(macSFReply);

		if gError.CheckOSError(theFile.Open(fsRdWrPerm)) then
			begin
										{ File was opened OK, now try to	}
										{   read bit map into memory. It's	}
										{   alright for this memory			}
										{   request to fail.					}

				gApplication.RequestMemory(FALSE, TRUE);
				theBitMap := theFile.ReadNewBitMap(TRUE);
				gApplication.RequestMemory(FALSE, FALSE);
				itsFile := theFile;

										{ Check that there was enough		}
										{   memory to open the Document	}

				if (theBitMap <> nil) and BuildWindow(theBitMap) then
					begin
										{ Make a window which displays	}
										{   the BitMap we just read in		}

										{ Set window title to the name of	}
										{   the file								}

						itsWindow.SetTitle(macSFReply.fName);

						itsWindow.Select;		{ Make this the active window		}

					end
				else
					begin						{ Not enough memory to open Doc	}
						gError.PostAlert(129, 2);
						Free;
					end;
			end;
	end;


{****************************************************}
{ BuildWindow}
{}
{		Create the window and panes for a painting document}
{}
{****************************************************}

	function CArtClassDoc.BuildWindow (theBitMap: CBitMap): Boolean;
		var
			theScrollPane: CScrollPane;
			thePaintPane: CPaintPane;
			bounds: Rect;
            theWindow: CWindow;         { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
        new(theWindow);             { Altered by TCL Weaver 1.0 (5/9/90) }
        itsWindow := theWindow;
		itsWindow.IWindow(WINDmyDoc, FALSE, gDesktop, SELF);
		SetRect(bounds, 100, 100, PNTG_WIDTH + SBARSIZE1, PNTG_HEIGHT + SBARSIZE1);
		itsWindow.SetSizeRect(bounds);
		gDecorator.PlaceNewWindow(itsWindow);

										{ Create a ScrollPane in the window }
		new(theScrollPane);
		theScrollPane.IViewRes('ScPn', 1, itsWindow, SELF);
		theScrollPane.FitToEnclFrame(TRUE, TRUE);

										{ Put our PaintPane inside the	}
										{   ScrollPane						}

		SetRect(bounds, 0, 0, PNTG_WIDTH, PNTG_HEIGHT);
		new(thePaintPane);
		gApplication.RequestMemory(FALSE, TRUE);
		thePaintPane.IPaintPane(theScrollPane, SELF, 0, 0, 0, 0, sizELASTIC, sizELASTIC, bounds, theBitMap);
		gApplication.RequestMemory(FALSE, FALSE);

		if thePaintPane.itsBitMap.macBitMap.baseAddr = nil then
			begin
				BuildWindow := FALSE;		{ Failed to create display			}
				Exit(BuildWindow);
			end

		else
			begin							{ Set up panes within window		}

				thePaintPane.FitToEnclosure(TRUE, TRUE);
				theScrollPane.InstallPanorama(thePaintPane);
				itsMainPane := thePaintPane;
				itsGopher := thePaintPane;

				theScrollPane.SetSteps(8, 8);			{ Set scrolling parameters			}
				theScrollPane.SetOverlaps(72, 72);
				BuildWindow := TRUE;
			end;
	end;



{*** F I L I N G   M E T H O D S ***}



{****************************************************}
{ DoSave (OVERRIDE)}
{}
{		Save a file under the same name.}
{}
{****************************************************}

	function CArtClassDoc.DoSave: Boolean;
		var
			theBitMap: CBitMap;

	begin
		if itsFile = nil then 				{ Same as SaveAs if there's no		}
			DoSave := DoSaveFileAs		{   existing file						}

		else
			begin
										{ Get pane ready to be saved		}

				CPaintPane(itsMainPane).KillSelRect;

										{ Write out bit map to disk as a	}
										{   MacPaint format file			}

				theBitMap := CPaintPane(itsMainPane).GetBitMap;
				CPNTGFile(itsFile).WriteBitMap(theBitMap);
				dirty := FALSE;
				DoSave := TRUE;
			end;
	end;


{****************************************************}
{ DoSaveAs (OVERRIDE)}
{}
{		Save a file under another name.}
{}
{****************************************************}

	function CArtClassDoc.DoSaveAs (macSFReply: SFReply): Boolean;
		var
			temp: OSErr;
            theFile: CFile;

	begin
		if itsFile <> nil then 				{ Throw out existing file object		}
			itsFile.Free;

                { Altered by TCL Weaver 1.0 (5/9/90) }

        new(CPNTGFile(theFile));            { Make a new file object with the   }
        itsFile := theFile;                 {   name from the SFReply record    }
        CPNTGFile(itsFile).IPNTGFile;
		itsFile.SFSpecify(macSFReply);
		temp := itsFile.CreateNew(gSignature, 'PNTG');
		temp := itsFile.Open(fsRdWrPerm);

		itsWindow.SetTitle(macSFReply.fName);

		DoSaveAs := DoSave;				{ Perform the actual write to disk	}
	end;


{****************************************************}
{ DoRevert (OVERRIDE)}
{}
{		Throw out the current contents of a document and revert to the}
{		last saved version.}
{}
{****************************************************}

	procedure CArtClassDoc.DoRevert;
		var
			theBitMap: CBitMap;
			homePos: Point;
			temp: OSErr;

	begin
		temp := itsFile.Close;			{ Close file and get rid of the	}
										{   offscreen bit image			}

		theBitMap := CPaintPane(itsMainPane).GetBitMap;
		theBitMap.Free;

		temp := itsFile.Open(fsRdWrPerm);			{ Reopen file and read original	}
													{   bit map from disk				}

		theBitMap := CPNTGFile(itsFile).ReadNewBitMap(TRUE);
		CPaintPane(itsMainPane).SetBitMap(theBitMap);

										{ Reset window to home position	}

		CPaintPane(itsMainPane).GetHomePosition(homePos);
		CPaintPane(itsMainPane).ScrollTo(homePos, FALSE);

										{ Get rid of any active selection		}

		SetRect(CPaintPane(itsMainPane).selRect, 0, 0, 0, 0);
		itsMainPane.Refresh;

		dirty := FALSE;
	end;


{****************************************************}
{ AboutToPrint}
{}
{		The specified range of pages is about to be printed}
{}
{****************************************************}

	procedure CArtClassDoc.AboutToPrint (var firstPage, lastPage: integer);
		var
			docName: Str255;

	begin
										{ Get pane ready to be printed		}

		CPaintPane(itsMainPane).KillSelRect;

		inherited AboutToPrint(firstPage, lastPage);

										{ Display banner to let user know	}
										{   printing is taking place			}

		PositionDialog('DLOG', DLOG_PRINT);
		printBanner := GetNewDialog(DLOG_PRINT, nil, WindowPtr(-1));
		DrawDialog(printBanner);

		GetName(docName);					{ Document name shown in status	}
		SetWTitle(printBanner, docName);	{   box when printing to a			}
											{   LaserWriter is taken from the	}
											{   title of the top window, which	}
											{   is now our dialog box.				}
	end;


{****************************************************}
{ DonePrinting (OVERRIDE)}
{}
{		Print loop has been completed}
{}
{****************************************************}

	procedure CArtClassDoc.DonePrinting;
		var
			itemType: integer;
			item: Handle;
			box: Rect;
			abortStr: Str255;
			ticks: longint;

	begin
		if PrError = iPrAbort then
			begin
				GetDItem(printBanner, 1, itemType, item, box);
				GetIndString(abortStr, STRcommon, strABORTPRINT);
				SetIText(item, abortStr);
				Delay(120, ticks);
			end;

		DisposDialog(printBanner);

		inherited DonePrinting;
	end;


end.