{****************************************************}
{}
{		CApplication.p																	}
{}
{		Implementation of the Application Class										}
{}
{		Abstract class for a Mac application. Programmers must create a subclass}
{		of CApplication for their own programs. The application is at the top of		}
{		the chain of command and defines the behavior of the program and its		}
{		interaction with the Mac OS.													}
{}
{		SUPERCLASS = CBureaucrat														}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}


unit CApplication;

interface

	uses
		SysEqu, TCL;


implementation


{** Class Constants **}

	const

		GROW_FAILURE = 0;		{ Return codes from GrowZoneFunc }
		GROW_SUCCESS = 1;

		JUMPBUFFER_A1 = 5;	{ Index of A1 in JumpBuffer }



{*****	I N I T I A L I Z A T I O N       M E T H O D S   ******}



{****************************************************}
{	IApplication }
{}
{		Initialize an Application. }
{}
{****************************************************}

	procedure CApplication.IApplication (extraMasters: integer; aRainyDayFund, aCreditLimit: Size);

{ Altered by TCL Weaver 1.0 (5/9/90) }

        var
            theSwitchboard: CSwitchboard;
            theDirectors: CCluster;
            theIdleChores: CList;
            theUrgentChores: CCluster;


	begin

		MenuDisable := LongPtr($B54);				{ Low-memory global	}

		nullStr := '';

		{ We haven't reached the event loop yet. 				}
		{ Flag A1 (jump addr) so we don't try to jump there. 	}

		eventLoopJump[JUMPBUFFER_A1] := 0;

		InitToolbox;

		InitMemory(extraMasters, aRainyDayFund, aCreditLimit);

		{ Instance Variables }

		IBureaucrat(nil);

            { Altered by TCL Weaver 1.0 (5/9/90) }

        new(theSwitchboard);
        new(theDirectors);
        new(theIdleChores);
        new(theUrgentChores);

        itsSwitchboard := theSwitchboard;
        itsDirectors := theDirectors;
        itsIdleChores := theIdleChores;
        itsUrgentChores := theUrgentChores;

        itsSwitchboard.ISwitchboard;
        itsDirectors.ICluster;
        itsIdleChores.IList;
		itsUrgentChores.ICluster;
		urgentsToDo := FALSE;
		running := TRUE;

		{ Global Variables }

		gSignature := '????';
		gHasWNE := WNEIsImplemented;
		gSleepTime := 0;						{ We want an early first Idle }
		new(gError);

		{ Cursors }

		gIBeamCursor := GetCursor(iBeamCursor);
		HNoPurge(Handle(gIBeamCursor));
		gWatchCursor := GetCursor(watchCursor);
		HNoPurge(Handle(gWatchCursor));

		gUtilRgn := NewRgn;

		MakeDesktop;
		MakeClipboard;
		MakeDecorator;
		SetUpFileParameters;
		SetUpMenus;

		gGopher := SELF;
		gLastViewHit := nil;
		gLastMouseUp.when := 0;
		gClicks := 0;
	end;


{****************************************************}
{ InitToolbox}
{}
{		Initialize the Macintosh Toolbox}
{}
{****************************************************}

	procedure CApplication.InitToolbox;

	begin

	{ Standard initialization calls	}

		InitGraf(@thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(nil);					{ ??? Add a ResumeProc		}
		InitCursor;
	end;


{****************************************************}
{ InitMemory}
{}
{		Initialize the Heap and create extra master pointer blocks}
{}
{****************************************************}

	procedure CApplication.InitMemory (extraMasters: integer; aRainyDayFund, aCreditLimit: Size);

	begin
		MaxApplZone;						{ Grow heap to maximum size to	}
										{   help prevent fragmentation		}

			{ Call MoreMasters to allocate extra master pointers.  		}
			{ Experiment to determine how many master pointers the 	}
			{ application uses under session of heavy use.					}

		while extraMasters > 0 do
			begin
				MoreMasters;
				extraMasters := extraMasters - 1;
			end;

		SetGrowZone(@GrowZoneFunc);		{ Traps out of memory conditions	}

		rainyDay := nil;			{ Clear rainyDay handle in case allocating the rainy day fund fails. }
								{ This is really bad. }

			{ Set instance variables }

		rainyDayFund := aRainyDayFund;
		creditLimit := aCreditLimit;
		rainyDayUsed := FALSE;
		memWarningIssued := FALSE;
		loanApproved := FALSE;
		canFail := FALSE;

		rainyDay := NewHandle(rainyDayFund);	{ Reserve a block of memory }

	end;


{****************************************************}
{ MakeDesktop}
{}
{		Create the global Desktop object, which is the top level of}
{		the visual hierarchy. Override this method to use a desktop}
{		which supports floating windows or other extensions to the}
{		standard desktop.}
{}
{****************************************************}

	procedure CApplication.MakeDesktop;

	begin
		new(gDesktop);				{ Use a standard Desktop			}
		gDesktop.IDesktop(SELF);
	end;


{****************************************************}
{ MakeClipboard}
{}
{		Create the global Clipboard object}
{}
{****************************************************}

	procedure CApplication.MakeClipboard;

	begin
		new(gClipboard);							{ Use standard Clipboard			}
		gClipboard.IClipboard(SELF, TRUE);
	end;


{****************************************************}
{ MakeDecorator}
{}
{		Create the global Decorator object, which controls the}
{		arrangement of windows on the screen. Override if a}
{		different Decorator is desired.}
{}
{****************************************************}

	procedure CApplication.MakeDecorator;

	begin
		new(gDecorator);			{ Use standard Decorator			}
		gDecorator.IDecorator;
	end;


{****************************************************}
{ SetUpFileParameters}
{}
{		Set values of the parameters used by the Standard File Package.}
{}
{****************************************************}

	procedure CApplication.SetUpFileParameters;

	begin
		sfNumTypes := -1;					{ Display all files				}
		sfFileTypes[0] := '????';			{ Just put something here	}
		sfFileFilter := nil;					{ No extra filtering of files	}

				{ GetFile Parameters }

		sfGetDLOGHook := nil;				{ No special handling of items	}
		sfGetDLOGid := getDlgID;			{ Use built-in dialog box			}
		sfGetDLOGFilter := nil;				{ No special event processing	}
	end;


{****************************************************}
{ SetUpMenus}
{}
{		Set up the menus used by the application}
{****************************************************}

	procedure CApplication.SetUpMenus;

	begin
		new(gBartender);
		gBartender.IBartender(MBARapp);
		AddResMenu(GetMHandle(MENUapple), 'DRVR');
		gBartender.SetDimOption(MENUapple, dimNONE);
	end;



{****	A C T I O N      M E T H O D S		****}



{****************************************************}
{ Notify (OVERRIDE)}
{}
{		A subordinate is notifying the Application that a task has been}
{		completed.}
{}
{****************************************************}

	procedure CApplication.Notify (theTask: CTask);

	begin
										{ NULL method	}
	end;


{****************************************************}
{ DoKeyDown (OVERRIDE)}
{}
{		Respond to a keystroke. Application has no supervisor to whom to}
{		pass the buck. Default action is to ignore keystrokes.}
{}
{****************************************************}

	procedure CApplication.DoKeyDown (theChar: char; keyCode: Byte; macEvent: EventRecord);

	begin
	end;										{ Null Method						}


{****************************************************}
{ DoAutoKey (OVERRIDE)}
{}
{		Respond to a key being held down. By default, keystrokes are ignored.}
{}
{****************************************************}

	procedure CApplication.DoAutoKey (theChar: char; keyCode: Byte; macEvent: EventRecord);

	begin
	end;										{ Null Method						}


{****************************************************}
{ DoKeyUp (OVERRIDE)}
{}
{		Respond to a key being released. Usually, these events will be}
{		masked out by the operating system. Ignore them by default.}
{}
{****************************************************}

	procedure CApplication.DoKeyUp (theChar: char; keyCode: Byte; macEvent: EventRecord);

	begin
	end;										{ Null Method						}


{****************************************************}
{ DoCommand}
{}
{		Handle a command. Application has no supervisor which can handle the}
{		command. If we don't handle the command here, it will never be}
{		executed (and that's an error).}
{}
{****************************************************}

	procedure CApplication.DoCommand (theCommand: longint);

		var
			macSFReply: SFReply;			{ Standard File reply record			}
			theDA: Str255;					{ Name of Desk Accessory to open	}
			tempInt: integer;
			tempBool: Boolean;

	begin
		if (theCommand < 0) then
			begin
				if (HiWord(-theCommand) = MENUapple) then
					begin
							{ Open a DA or launch another	}
							{   application under MultiFinder	}

						GetItem(GetMHandle(MENUapple), LoWord(-theCommand), theDA);
						tempInt := OpenDeskAcc(theDA);
					end;
			end
		else
			begin

				case theCommand of

					cmdNew: 
						begin
							SetCursor(gWatchCursor^^);
							CreateDocument;
						end;

					cmdOpen: 
						begin
							ChooseFile(macSFReply);
							if macSFReply.good then
								begin
									SetCursor(gWatchCursor^^);
									OpenDocument(macSFReply);
								end;
						end;

					cmdClose: 
						if IsSystemWindow(WindowPeek(FrontWindow)) then
							CloseDeskAcc(WindowPeek(FrontWindow)^.windowKind);

					cmdQuit: 
						Quit;

					cmdUndo, cmdCut, cmdCopy, cmdPaste, cmdClear: 
						tempBool := SystemEdit(integer(theCommand - cmdUndo));

					cmdToggleClip: 
						gClipboard.Toggle;

					otherwise
						;				{ You may want to signal an error here }
							{ for debugging purposes. }

				end;
			end;
	end;


{****************************************************}
{ UpdateMenus (OVERRIDE)}
{}
{		Perform menu management tasks}
{}
{****************************************************}

	procedure CApplication.UpdateMenus;

		var
			undoStr: Str255;

	begin
		gBartender.EnableCmd(cmdQuit);

		if gInBackground then
			begin
				gBartender.EnableCmd(cmdClose);
				gBartender.EnableCmd(cmdUndo);
				gBartender.EnableCmd(cmdCut);
				gBartender.EnableCmd(cmdCopy);
				gBartender.EnableCmd(cmdPaste);
				gBartender.EnableCmd(cmdClear);
			end
		else
			gBartender.EnableCmd(cmdToggleClip);

		if not rainyDayUsed then
			begin
				gBartender.EnableCmd(cmdNew);
				gBartender.EnableCmd(cmdOpen);
			end;

		GetIndString(undoStr, STRcommon, strUNDO);
		gBartender.SetCmdText(cmdUndo, undoStr);
	end;



{*** M E M O R Y   M A N A G E M E N T ***}



{****************************************************}
{ RequestMemory}
{}
{ 		Turn on/off the flags which indicate whether subsequent memory}
{ 		requests can use the rainy day fund and can fail}
{}
{****************************************************}

	procedure CApplication.RequestMemory (aLoanApproved, aCanFail: Boolean);

	begin
		loanApproved := aLoanApproved;
		canFail := aCanFail;
	end;


{****************************************************}
{	 GrowMemory}
{}
{	        A memory request can't be filled by the Mac Memory Manager. }
{	        Free up memory if possible or gracefully shutdown.}
{}
{	        A rainy day fund of reserve memory is maintained to monitor low }
{	        memory conditions. If the application can't free up enough memory}
{	        on its own, we use the reserve fund. The user is notified when}
{	        we tap into the reserve fund so that he/she can take steps to free}
{	        up memory (perhaps by closing windows/files).}
{}
{	        If there is not enough memory in reserve to cover a request, we}
{	        notify the user of impending doom. The Toolbox Memory Manager will}
{	        return an error code of memFullErr. This will probably cause some kind}
{	        of System Bomb.}
{}
{****************************************************}

	function CApplication.GrowMemory (bytesNeeded: Size): longint;

		var
			bytesToFree, rainyDayBalance, grow: Size;

	begin
		MemoryShortage(bytesNeeded);
		bytesToFree := bytesNeeded - MaxMem(grow);
		if bytesToFree < 1 then
			GrowMemory := GROW_SUCCESS
		else if (rainyDay <> nil) and (loanApproved or (bytesToFree <= creditLimit)) then
			begin
				rainyDayBalance := GetHandleSize(rainyDay);
				if rainyDayBalance - bytesToFree >= MINIMUM_BALANCE then
					SetHandleSize(rainyDay, MINIMUM_BALANCE)
				else if rainyDayBalance >= bytesToFree then
					begin
						DisposHandle(rainyDay);
						rainyDay := nil;
					end
				else
					begin
						DisposHandle(rainyDay);
						rainyDay := nil;
					end;
				gSleepTime := 0;
				rainyDayUsed := TRUE;
				GrowMemory := GROW_SUCCESS;
			end
		else if canFail then
			GrowMemory := GROW_FAILURE
		else
			GrowMemory := OutOfMemory(bytesNeeded);
	end;


{****************************************************}
{	 MemoryShortage}
{}
{	        There is not enough memory to fill a request. Try to free up some}
{	        memory. Subclasses should override this method. }
{}
{	        The application should free all non-essential blocks of}
{	        memory (such as those used to cache data for speed or resources}
{	        marked as unpurgeable to allow for quick access) and disable}
{	        commands which could consume a lot of memory (such as opening files}
{	        or creating new documents). The rainy day fund created by InitMemory}
{	        should be large enough to allow (or even force) the user to take}
{	        actions to reduce the memory strain without losing data (such as}
{	        closing windows/files).}
{}
{	        IMPORTANT: This method should NOT allocate any blocks of memory.}
{}
{****************************************************}

	procedure CApplication.MemoryShortage (bytesNeeded: Size);

	begin
	end;


{****************************************************}
{	 MemoryReplenished}
{}
{	        The rainy day fund of reserve memory has been replenished.}
{	        Subclasses must override this method if certain actions taken}
{	        by the MemoryShortage method need to be undone. For example, if}
{	        the "Open..." command was disabled by MemoryShortage, it should}
{	        be enabled here.}
{}
{****************************************************}

	procedure CApplication.MemoryReplenished;

	begin
	end;


{****************************************************}
{ OutOfMemory}
{}
{ 		A memory request cannot be satisfied. At this point, the rainy day}
{ 		fund has been liquidated. This method posts an Alert if there's}
{ 		enough memory to do so, and then jumps back to the event loop.}
{ 		Classes which override this method should pass back the appropriate}
{ 		return value (same as that for a GrowZone function) if necessary.}
{ 		This method performs a long jump, so no return value is needed.}
{}
{****************************************************}

	function CApplication.OutOfMemory (bytesNeeded: Size): longint;

		var
			grow: Size;
			temp: Boolean;

	begin
		if MaxMem(grow) > MINIMUM_BALANCE then
			temp := gError.CheckOSError(memFullErr);

		HiliteMenu(NOTHING);
		JumpToEventLoop;
	end;



{****	E X E C U T I O N     M E T H O D S 	****}



{****************************************************}
{ Run}
{}
{		Run the application by processing events. }
{}
{****************************************************}

	procedure Chore_Perform (theChore: CChore; minSleepPtr: Ptr);

		var
			maxSleep: longint;

	begin
		maxSleep := maxlongint;
		theChore.Perform(maxSleep);
		longPtr(minSleepPtr)^ := Min(longPtr(minSleepPtr)^, maxSleep);
	end;


	procedure CApplication.Run;
		var
			aDAIsActive: Boolean;			{ Is a DA the front window?		}
			minSleep: longint;
			temp: integer;

	begin
		minSleep := 0;

		Preload;		{ Open/Print files selected from the Finder before 	}
					{ launching application 									}

{$IFC ResumeAfterError}
		temp := f_SetJump(eventLoopJump);
{$ENDC}

		aDAIsActive := IsSystemWindow(WindowPeek(FrontWindow));

		repeat
			begin
				gDesktop.Cleanup;	{ Make sure our top window is in front of 	}
								{ THINK Pascal's windows. 					}

				itsSwitchboard.ProcessEvent;

				if urgentsToDo then
					begin
						itsUrgentChores.DoForEach1(Chore_Perform, @minSleep);
						itsUrgentChores.DisposeItems;
						urgentsToDo := FALSE;
					end;

{	Check for context-switch with Desk Accessories		}

				if IsSystemWindow(WindowPeek(FrontWindow)) then
					begin
						gSleepTime := 0;
						if not aDAIsActive then
							begin
								aDAIsActive := TRUE;
								SwitchToDA;
							end;
					end
				else
					begin
						if aDAIsActive then
							begin
								aDAIsActive := FALSE;
								SwitchFromDA;
							end;
					end;
			end
		until not running;
	end;


{****************************************************}
{ Preload}
{}
{		Open/Print files selected by the user from the Finder before}
{		launching the application}
{}
{****************************************************}

	procedure CApplication.Preload;

		var
			openOrPrint: integer;			{ Type of action requested	}
			numPreloads: integer;			{ No. of files to process		}
			i: integer;						{ Index for files				}
			macAppFile: AppFile;			{ Info about a file				}
			macSFReply: SFReply;			{ Standard File info record	}

	begin

{ Check if files were selected		}

		CountAppFiles(openOrPrint, numPreloads);

{ Process files one by one			}

		for i := 1 to numPreloads do
			begin

{ Get info about the file				}

				GetAppFiles(i, macAppFile);

{ Copy info into a Standard File record. }

				macSFReply.fType := macAppFile.fType;
				macSFReply.vRefNum := macAppFile.vRefNum;
				macSFReply.version := macAppFile.versNum;
				macSFReply.fName := macAppFile.fName;

{ Automatically open selected file	}

				OpenDocument(macSFReply);

				if openOrPrint = appPrint then  { User wants to print Document }
					begin
						gDesktop.UpdateWindows;
						gGopher.DoCommand(cmdPrint);
					end;

				ClrAppFiles(i);					{ We've processed this file		}
			end;

		StartUpAction(numPreloads);		{ Perform start up action		}
	end;


{****************************************************}
{ StartUpAction}
{}
{		Perform any desired start up action. This method is invoked after}
{		files selected from the Finder have been opened/printed. The number}
{		of preloaded files is passed as a parameter. If no files were}
{		preloaded, this default method acts as if the user selected "New"}
{		and sends the Gopher a DoCommand(cmdNew) message.}
{}
{****************************************************}

	procedure CApplication.StartUpAction (numPreloads: integer);
	begin
		FlushEvents(everyEvent, 0);
		if numPreloads = 0 then
			gGopher.DoCommand(cmdNew);
	end;


{****************************************************}
{ Suspend}
{}
{		Program is about to be swapped out under MultiFinder. Send the}
{		Suspend message to each of the Application's Directors.}
{}
{****************************************************}

	procedure CApplication.Suspend;
	begin
					{ Tell all our directors that we	}
					{   are about to lose control		}

		itsDirectors.DoForEach(Director_Suspend);

		gInBackground := TRUE;			{ We are going into the background	}
	end;


{****************************************************}
{ Resume}
{}
{		Program is being swapped in under MultiFinder}
{}
{****************************************************}

	procedure CApplication.Resume;
	begin

{ Do nothing if a DA is in front	}

		if not IsSystemWindow(WindowPeek(FrontWindow)) then
			begin
				gInBackground := FALSE;	{ We are no longer in background	}

					{ Tell all directors we are back	}
					{   in control						}

				itsDirectors.DoForEach(Director_Resume);
			end;
	end;


{****************************************************}
{ SwitchToDA}
{}
{		A DA is becoming active}
{}
{****************************************************}

	procedure CApplication.SwitchToDA;
	begin
		LongPtr(CurDeactive)^ := longint(nil);
		Suspend;
	end;


{****************************************************}
{ SwitchFromDA}
{}
{		Our application is becoming active after a DA had been running}
{}
{****************************************************}

	procedure CApplication.SwitchFromDA;
	begin
		LongPtr(CurActivate)^ := longint(nil);
		Resume;
	end;


{****************************************************}
{ Quit}
{}
{		User requested quit. Need to perform a graceful shutdown,}
{		giving the user the opportunity to save changed files or to abort}
{		the quit process.}
{}
{****************************************************}

	procedure CApplication.Quit;
		var
			theWindow: CWindow;
			theDirector: CDirector;

	begin
		running := FALSE;		{ Assume we will stop running	}

{	Run through all our windows (doesn't include floating windows or DAs) }

		theWindow := gDesktop.GetTopWindow;
		while theWindow <> nil do
			begin
				theDirector := CDirector(theWindow.GetSupervisor);
				if not theDirector.Quit then
					begin
						running := TRUE;
						Leave;
					end;
				theWindow := gDesktop.GetTopWindow;
			end;
	end;


{****************************************************}
{ Exit}
{}
{		Program is about to terminate. Last chance to clean up.}
{}
{****************************************************}

	procedure CApplication.Exit;
	begin
	end;										{ NULL Method						}


{****************************************************}
{ JumpToEventLoop}
{}
{		Jump to the beginning of the main event loop. Useful when trying}
{		to recover from an error.}
{}
{		An error may occur BEFORE program ever gets to the event loop. In}
{		this case the jump buffer entry for A1 (the address to jump to) will}
{		be nil and we exit the program.}
{****************************************************}

	procedure CApplication.JumpToEventLoop;
	begin
		if eventLoopJump[JUMPBUFFER_A1] <> 0 then
			f_LongJump(eventLoopJump, 1)
		else
			begin			{ Either we never got to the event loop, or the	}
				Exit;			{ ResumeAfterError compile-time var = FALSE.	}
				ExitToShell;	{ Not much we can do except abort the program. 	}
			end;
	end;



{*** D O C U M E N T   H A N D L I N G   M E T H O D S ****}



{****************************************************}
{ CreateDocument}
{}
{		Make a document in response to the "New" menu selection. This method}
{		must be overridden if the standard "New" command is used.}
{}
{****************************************************}

	procedure CApplication.CreateDocument;
	begin
	end;										{ NULL Method						}


{****************************************************}
{ OpenDocument}
{}
{		Open an existing file and create a document object for displaying}
{		information. This method must be overridden if the standard "Open"}
{		command is used.}
{}
{****************************************************}

	procedure CApplication.OpenDocument (macSFReply: SFReply);
	begin
	end;										{ NULL Method						}


{****************************************************}
{ ChooseFile}
{}
{		Let the user select a file to open. Default procedure is to use}
{		the Standard GetFile dialog box.}
{}
{****************************************************}

	procedure CApplication.ChooseFile (var macSFReply: SFReply);
		var
			corner: Point;				{ Top left corner of dialog box }
			saveHState: SignedByte;

	begin
		FindDlogPosition('DLOG', sfGetDLOGid, corner);

		saveHState := HGetState(Handle(SELF));
		HLock(Handle(SELF));

		SFPGetFile(corner, nullStr, sfFileFilter, sfNumTypes, sfFileTypes, sfGetDLOGHook, macSFReply, sfGetDLOGid, sfGetDLOGFilter);

		HSetState(Handle(SELF), saveHState);
	end;


{****************************************************}
{ AddDirector}
{}
{		Add a Director to an Application's list of Directors}
{}
{****************************************************}

	procedure CApplication.AddDirector (aDirector: CDirector);
	begin
		itsDirectors.Add(aDirector);
	end;


{****************************************************}
{ RemoveDirector}
{}
{		Eliminate a Director}
{}
{****************************************************}

	procedure CApplication.RemoveDirector (theDirector: CDirector);
	begin
		itsDirectors.Remove(theDirector);
	end;



{**** C H O R E    H A N D L I N G    M E T H O D S ****}



{****************************************************}
{	 Idle}
{}
{	        No event directed to our application is pending. Here's our chance}
{	        to perform periodic tasks. The classic example is flashing a}
{	        text insertion cursor.}
{}
{****************************************************}

	procedure CApplication.Idle (macEvent: EventRecord);
		var
			theBureaucrat: CBureaucrat;		{ Object in the chain of command 	}
			grow: Size;							{ Amount heap may be grown 		}
			minSleep, maxSleep: longint;

	begin
		if rainyDayUsed then	{ Memory reserve needs to be replenished	}

			if MaxMem(grow) > rainyDayFund + MINIMUM_BALANCE then

				begin						{ Enough memory has been freed		}
					if rainyDay <> nil then
						DisposHandle(rainyDay);
					rainyDay := NewHandle(rainyDayFund);
					rainyDayUsed := FALSE;
					memWarningIssued := FALSE;
					MemoryReplenished;
				end

			else if not memWarningIssued then	{ Inform user of memory shortage }
				begin
					gError.PostAlert(STRmemWarn, iMEM_LOW);
					memWarningIssued := TRUE;
				end;
										{ Idle time for objects in the chain of command }
		theBureaucrat := gGopher;		{ Start at the bottom								 }
		minSleep := maxlongint;
		repeat
			begin							{ Let bureaucrats kill some time }
				maxSleep := maxlongint;
				theBureaucrat.Dawdle(maxSleep);
				minSleep := Min(minSleep, maxSleep);
				theBureaucrat := theBureaucrat.itsSupervisor;
			end
		until theBureaucrat = nil;

		itsIdleChores.DoForEach1(Chore_Perform, @minSleep);		{ Do chores }
		gSleepTime := minSleep;
	end;


{****************************************************}
{	 AssignIdleChore}
{}
{	        Give Application a chore to perform at idle time. Idle chores are}
{	        performed whenever there are no events pending for the Application.}
{}
{****************************************************}

	procedure CApplication.AssignIdleChore (theChore: CChore);
	begin
		itsIdleChores.Add(theChore);
		gSleepTime := 0;					{ Force immediate idle }
	end;


{****************************************************}
{	 CancelIdleChore}
{}
{	        Relieve Application of an idle time chore. A chore must NOT cancel}
{	        itself or any other chore.}
{}
{****************************************************}

	procedure CApplication.CancelIdleChore (theChore: CChore);
	begin
		itsIdleChores.Remove(theChore);
	end;


{****************************************************}
{	 AssignUrgentChore}
{}
{	        Give Application a chore to perform as soon as possible. Urgent}
{	        chores are performed only once, then automatically disposed.}
{	        An urgent chore must NOT post another urgent chore.}
{}
{****************************************************}

	procedure CApplication.AssignUrgentChore (theChore: CChore);
	begin
		itsUrgentChores.Add(theChore);
		urgentsToDo := TRUE;
	end;


end.