{****************************************************}
{}
{		 CButton.p}
{}
{		Implementation of the Button Class}
{}
{		A standard Mac push-button control.  It can issue a }
{		command to its supervisor after it is clicked. }
{}
{		SUPERCLASS = CControl}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CButton;

interface

	uses
		TCL;


implementation



{**** C O N S T R U C T I O N    M E T H O D S ****}



{****************************************************}
{ IButton}
{}
{		Initialize a Button object}
{}
{****************************************************}

	procedure CButton.IButton (CNTLid: integer; anEnclosure: CView; aSupervisor: CBureaucrat);
		var
            enclOriginH, enclOriginV: longint;  { Altered by TCL Weaver 1.0 (4/27/90) }
			macControlP: ControlPtr;

	begin
		IView(anEnclosure, aSupervisor);

		macPort := itsEnclosure.GetMacPort;
		macControl := GetNewControl(CNTLid, macPort);
		CheckResource(Handle(macControl));
		macControlP := macControl^;
		clickCmd := cmdNull;

		visible := (macControlP^.contrlVis = 255);
		active := TRUE;
		wantsClicks := TRUE;

		hEncl := macControlP^.contrlRect.left;
		vEncl := macControlP^.contrlRect.top;
		width := macControlP^.contrlRect.right - hEncl;
		height := macControlP^.contrlRect.bottom - vEncl;
		hSizing := sizFIXEDSTICKY;
		vSizing := sizFIXEDSTICKY;
		autoRefresh := TRUE;
		printClip := clipFRAME;
		printing := FALSE;

        anEnclosure.GetOrigin(enclOriginH, enclOriginV);    { Altered by TCL Weaver 1.0 (4/27/90) }
        hEncl := hEncl + enclOriginH;
        vEncl := vEncl + enclOriginV;

		CalcFrame;					{ Set up Frame coords }
		CalcAperture;				{ Determine visible portion }

		itsEnvironment := nil;
		itsEnclosure.AddSubview(SELF);		{ This is a subview of enclosure	}
	end;



{**** C L I C K    H A N D L I N G    M E T H O D S ****}



{****************************************************}
{ DoGoodClick (OVERRIDE)}
{}
{		A Button responds to a good click (the mouse being pressed}
{		and released within the button) by passing a particular command}
{		to its supervisor.}
{}
{****************************************************}

	procedure CButton.DoGoodClick (whichPart: integer);
	begin
		if clickCmd <> cmdNull then				{ Issue the appropriate command	}
			itsSupervisor.DoCommand(clickCmd);
	end;


{****************************************************}
{ SetClickCmd}
{}
{		Specify the command which is sent to a Button's supervisor}
{		after a confirmed click}
{}
{****************************************************}

	procedure CButton.SetClickCmd (aClickCmd: longint);
	begin
		clickCmd := aClickCmd;				{ Set instance variable			}
	end;

end.