{****************************************************}
{ 		CEditText.p}
{}
{		The EditText Class}
{}
{		SUPERCLASS = CStaticText}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CEditText;

interface

	uses
		TCL;

	function TEClickLoop: Boolean;


implementation


{****************************************************}
{ TEClickLoop}
{}
{		Function for auto-scrolling text, called through a pointer}
{}
{****************************************************}

	function TEClickLoop: Boolean;
		var
			mouseLoc: Point;
			temp: Boolean;

	begin
		if gEditText <> nil then
			begin
				GetMouse(mouseLoc);
				temp := gEditText.AutoScroll(mouseLoc);
			end;
		TEClickLoop := TRUE;
	end;



{*** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ***}



{****************************************************}
{ IEditText}
{}
{		Initialize a EditText object}
{}
{****************************************************}

	procedure CEditText.IEditText (anEnclosure: CView; aSupervisor: CBureaucrat; aWidth, aHeight, aHEncl, aVEncl: integer; aHSizing, aVSizing: SizingOption; aLineWidth: integer);
		var
			itemStr: Str255;

	begin
		IStaticText(anEnclosure, aSupervisor, aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing, aLineWidth);

		wantsClicks := TRUE;
		SetClikLoop(@TEClickLoop, macTE);

		GetFontName(macTE^^.txFont, itemStr);
		fontItem := gBartender.FindItemText(MENUfont, itemStr);

		if macTE^^.txSize = 0 then
			macTE^^.txSize := SYS_FONT_SIZE;
		NumToString(macTE^^.txSize, itemStr);
		sizeItem := gBartender.FindItemText(MENUsize, itemStr);
	end;



{*** C O M M A N D   M E T H O D S ***}



{****************************************************}
{ DoCommand (OVERRIDE)}
{}
{		Execute a command}
{}
{****************************************************}

	procedure CEditText.DoCommand (theCommand: longint);
		var
			itemName: Str255;
			number, temp: longint;

	begin
		if theCommand < 0 then
			begin

				case HiWord(-theCommand) of

					MENUfont: 
						begin
							gBartender.GetCmdText(theCommand, itemName);
							fontItem := LoWord(-theCommand);
							SetFontName(itemName);
						end;

					MENUsize: 
						begin
							gBartender.GetCmdText(theCommand, itemName);
							StringToNum(itemName, number);
							sizeItem := LoWord(-theCommand);
							SetFontSize(integer(number));
						end;

					otherwise
						inherited DoCommand(theCommand);

				end;

			end
		else
			begin

				case theCommand of

					cmdCut: 
						begin
							Prepare;
							TECut(macTE);
							temp := ZeroScrap;
							temp := TEToScrap;
							AdjustBounds;
							ScrollToSelection;
							gClipboard.UpdateDisplay;
						end;

					cmdCopy: 
						begin
							TECopy(macTE);
							temp := ZeroScrap;
							temp := TEToScrap;
							gClipboard.UpdateDisplay;
						end;

					cmdPaste: 
						begin
							Prepare;
							temp := TEFromScrap;
							TEPaste(macTE);
							AdjustBounds;
							ScrollToSelection;
						end;

					cmdClear: 
						begin
							Prepare;
							TEDelete(macTE);
							AdjustBounds;
							ScrollToSelection;
						end;

					cmdPlain: 
						SetFontStyle([]);

					cmdBold: 
						SetFontStyle([bold]);

					cmdItalic: 
						SetFontStyle([italic]);

					cmdUnderline: 
						SetFontStyle([underline]);

					cmdOutline: 
						SetFontStyle([outline]);

					cmdShadow: 
						SetFontStyle([shadow]);

					cmdCondense: 
						SetFontStyle([condense]);

					cmdExtend: 
						SetFontStyle([extend]);

					cmdAlignLeft,				{ Good numbering lets us subtract	}
					cmdAlignCenter,			{   to set alignment, left = 0,		}
					cmdAlignRight:				{   center = 1, right = -1			}

						SetAlignment(theCommand - cmdAlignLeft);

					cmdSingleSpace, cmd1HalfSpace, cmdDoubleSpace: 
						SetLineSpacing(theCommand);

					otherwise
						inherited DoCommand(theCommand);
				end;
			end;
	end;


{****************************************************}
{ UpdateMenus (OVERRIDE)}
{}
{		Perform menu management tasks}
{}
{****************************************************}

	procedure CEditText.UpdateMenus;
		var
			macMenu: MenuHandle;
			theStyle: Style;
			macTEP: TEPtr;

	begin
		inherited UpdateMenus;
		macTEP := macTE^;
		if macTEP^.selStart <> macTEP^.selEnd then
			begin
				gBartender.EnableCmd(cmdCut);
				gBartender.EnableCmd(cmdCopy);
				gBartender.EnableCmd(cmdClear);
			end;

		if gClipboard.DataSize('TEXT') > 0 then
			gBartender.EnableCmd(cmdPaste);

		if fontItem > 0 then
			begin
				macMenu := GetMHandle(MENUfont);
				if macMenu <> nil then
					CheckItem(macMenu, fontItem, TRUE);
			end;

		if sizeItem > 0 then
			begin
				macMenu := GetMHandle(MENUsize);
				if macMenu <> nil then
					CheckItem(macMenu, sizeItem, TRUE);
			end;

		theStyle := macTE^^.txFace;
		if theStyle = [] then
			gBartender.CheckMarkCmd(cmdPlain, TRUE)

		else
			begin
				if bold in theStyle then
					gBartender.CheckMarkCmd(cmdBold, TRUE);

				if italic in theStyle then
					gBartender.CheckMarkCmd(cmdItalic, TRUE);

				if underline in theStyle then
					gBartender.CheckMarkCmd(cmdUnderline, TRUE);

				if outline in theStyle then
					gBartender.CheckMarkCmd(cmdOutline, TRUE);

				if shadow in theStyle then
					gBartender.CheckMarkCmd(cmdShadow, TRUE);

				if condense in theStyle then
					gBartender.CheckMarkCmd(cmdCondense, TRUE);

				if extend in theStyle then
					gBartender.CheckMarkCmd(cmdExtend, TRUE);

			end;

		gBartender.CheckMarkCmd(macTE^^.just + cmdAlignLeft, TRUE);

		gBartender.CheckMarkCmd(spacingCmd, TRUE);
	end;



{*** M O U S E   A N D   K E Y S T R O K E   M E T H O D S ***}



{****************************************************}
{ DoClick (OVERRIDE)}
{}
{		Respond to a mouse click within the EditText}
{}
{****************************************************}

	procedure CEditText.DoClick (hitPt: Point; modifierKeys: integer; when: longint);
	begin

{   Altered by TCL Weaver 1.0 (4/27/90) }

        if gEditText <> SELF then
            begin
                if gEditText <> nil then
                    gEditText.Deactivate;
                Activate;
                gGopher := SELF;
            end;

		TEClick(hitPt, BAND(modifierKeys, shiftKey) <> NOTHING, macTE);
	end;


{****************************************************}
{ DoKeyDown (OVERRIDE)}
{}
{		Handle a keystroke}
{}
{****************************************************}

	procedure CEditText.DoKeyDown (theChar: char; keyCode: Byte; macEvent: EventRecord);
		var
			thePosition: Point;
			theHExtent, theVExtent: longint;

	begin
		case keyCode of

			KeyHome, KeyPageUp, KeyPageDown: 
				inherited DoKeyDown(theChar, keyCode, macEvent);

			KeyEnd: 
				if itsScrollPane <> nil then
					begin
						GetExtent(theHExtent, theVExtent);
						thePosition.h := 0;
						thePosition.v := Max(0, theVExtent - itsScrollPane.vSpan);
						ScrollTo(thePosition, TRUE);
						itsScrollPane.Calibrate;
					end;

			otherwise
				begin
					Prepare;
					TEKey(theChar, macTE);
					AdjustBounds;
					ScrollToSelection;
				end;
		end;
	end;


{****************************************************}
{ DoAutoKey (OVERRIDE)}
{}
{		Handle a key being held down. Treat this the same as a normal}
{		keystroke.}
{}
{****************************************************}

	procedure CEditText.DoAutoKey (theChar: char; keyCode: Byte; macEvent: EventRecord);
	begin
		if BAND(macEvent.modifiers, cmdKey) = 0 then
			DoKeyDown(theChar, keyCode, macEvent);
	end;



{*** D I S P L A Y   M E T H O D S ***}



{****************************************************}
{ Activate (OVERRIDE)}
{}
{		Activate EditText by hilighting the selection or showing the}
{		text insertion caret. Also enable applicable menu commands.}
{}
{****************************************************}

	procedure CEditText.Activate;
	begin
		inherited Activate;

		Prepare;
		TEActivate(macTE);
		gEditText := SELF;
	end;


{****************************************************}
{ Deactivate (OVERRIDE)}
{}
{		Deactivate EditText by unhiliting the selection or hiding the}
{		text insertion caret. Also disable the appropriate menu commands.}
{}
{****************************************************}

	procedure CEditText.Deactivate;
	begin
		inherited Deactivate;

		Prepare;
		TEDeactivate(macTE);
		gEditText := nil;
	end;



{*** P R I N T I N G   M E T H O D S ***}



{****************************************************}
{ AboutToPrint (OVERRIDE)}
{}
{		The specified range of pages is about to be printed.}
{}
{****************************************************}

	procedure CEditText.AboutToPrint (var firstPage, lastPage: integer);
	begin
		if active then
			begin
				Prepare;						{ Must make sure that the		}
				HidePen;						{   current selection is not		}
				TEDeactivate(macTE);		{   hilited on the print out		}
				ShowPen;
			end;

		inherited AboutToPrint(firstPage, lastPage);
	end;


{****************************************************}
{ DonePrinting (OVERRIDE)}
{}
{		Print loop has been completed. Re-hilite the active selection.}
{}
{****************************************************}

	procedure CEditText.DonePrinting;
	begin
		inherited DonePrinting;

		if active then
			begin						{ Restore active state				}
				Prepare;
				HidePen;
				TEActivate(macTE);
				ShowPen;
			end;
	end;



{*** C U R S O R   M E T H O D S ***}



{****************************************************}
{ Dawdle (OVERRIDE)}
{}
{		The user isn't doing anything, so flash the text insertion cursor}
{}
{****************************************************}

	procedure CEditText.Dawdle (var maxSleep: longint);
	begin
		Prepare;
		TEIdle(macTE);
		maxSleep := GetCaretTime;
	end;


{****************************************************}
{ AdjustCursor (OVERRIDE)}
{}
{		Mouse is inside an EditText Pane. Use the standard IBeam cursor.}
{}
{****************************************************}

	procedure CEditText.AdjustCursor (where: Point; mouseRgn: RgnHandle);
	begin
		SetCursor(gIBeamCursor^^);
	end;


end.