{****************************************************}
{ 		CError.p}
{}
{		The Error Class}
{}
{		Error handling class for reporting errors.}
{}
{		SUPERCLASS = CObject}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CError;

interface

	uses
		Script, TCL;


implementation



{****************************************************}
{	 SevereMacError }
{}
{	        Report a Macintosh Operating System Error by posting an Alert }
{	        Box which gives the user the choice of continuing at his/her own }
{	        risk or aborting the program. }
{}
{****************************************************}

	procedure CError.SevereMacError (macErr: OSErr);
		var
			errMsg: StringHandle;		{ Text of error message }
			errNum: Str255;			{ Error number as a string }
            errorString: Str255;        { Altered by TCL Weaver 1.0 (4/27/90) }

	begin
		errMsg := StringHandle(GetResource(ErrMsg_Res, macErr));
		if errMsg = nil then
			errMsg := GetString(STRosError);
		if errMsg = nil then
			if macErr = resNotFound then
				MissingResources
			else
                begin                       { Altered by TCL Weaver 1.0 (4/27/90) }
                    errorString := 'Mac OS Error encountered';
                    errMsg := StringHandle(NewHandle(length(errorString) + 1));
                    errMsg^^ := errorString;
                end;
        NumToString(macErr, errNum);
        HLock(Handle(errMsg));
        ParamText(errMsg^^, errNum, '', '');
        HUnlock(Handle(errMsg));
        PositionDialog('ALRT', ALRTsevereErr);
        InitCursor;
        if StopAlert(ALRTsevereErr, nil) = Cancel then
            begin
                gApplication.Exit;
                ExitToShell;
            end
        else
            begin
                HiliteMenu(NOTHING);
                gApplication.JumpToEventLoop;
			end;
	end;


{****************************************************}
{	 CheckOSError }
{}
{	        Check for a Macintosh Operating System Error and post an Alert }
{	        if necessary. Return TRUE if everything is OK; Return FALSE }
{	        if an error occurred. }
{}
{	        Method looks for text for an error message in an ErrMsg_Res resource }
{	        with the same ID number as the error number. If it doesn't find }
{	        one, it uses the default error message which is stored in an }
{	        'STR ' resource with the same ID as that used for the Alert Box. }
{}
{****************************************************}

	function CError.CheckOSError (macErr: OSErr): Boolean;
		var
			errMsg: StringHandle;
			errNum: Str255;
			temp: integer;

	begin
		if macErr = noErr then
			CheckOSError := TRUE
		else
			begin
				errMsg := StringHandle(GetResource(ErrMsg_Res, macErr));
				if errMsg = nil then
					errMsg := GetString(STRosError);
				NumToString(macErr, errNum);
				HLock(Handle(errMsg));
				ParamText(errMsg^^, errNum, '', '');
				HUnlock(Handle(errMsg));
				PositionDialog('ALRT', ALRTosError);
				InitCursor;
				temp := StopAlert(ALRTosError, nil);
				CheckOSError := FALSE;
			end;
	end;


{****************************************************}
{ PostAlert}
{}
{		Post a standard Mac Alert box with a text string specified by}
{		a string list resource ID (STR#) and index number}
{}
{****************************************************}

	procedure CError.PostAlert (STRid, index: integer);
		var
			message: Str255;
			temp: integer;

	begin
		GetIndString(message, STRid, index);
		ParamText(message, '', '', '');
		PositionDialog('ALRT', ALRTgeneral);
		InitCursor;
		temp := Alert(ALRTgeneral, nil);
	end;


{****************************************************}
{ MissingResources }
{}
{		We tried to load two non-existent resources.  The user probably forgot to }
{		specify a resource file in Pascal's Run Options dialog. }
{}
{		Post an alert and terminate the application. }
{}
{****************************************************}

	procedure CError.MissingResources;

		type										{ Template for a DITL consisting of one 	}
			textItem = record					{ static text item.							}
					numItems: integer;
					itemHandle: Handle;
					itemRect: Rect;
					itemTypeAndLength: integer;		{ Type in high byte, length in low byte.	}
					itemContent: Str255;				{ Actually, this needs to be pure text	}
												{ (no length byte).  Also, the number of	}
												{ chars must be even.						}
				end;
			textItemP = ^textItem;
			textItemH = ^textItemP;


		const										{ Constants used in setting up the DITL.	}

			theText = 'ome resources were not found.  Did you specify a resource file in the Run Options dialog?';

												{ We'll stuff the length byte with an 'S' 	}
												{ later.										}

			theTextLength = length(theText) + 1;
			textItemType = (statText + itemDisable) * $0100;

			TextTopLeft = 20;
			DialogRight = 255;
			DialogBottom = 105;

		var
			theDialogTextP: textItemP;
			theDialogText: textItemH;
			theDialog: DialogPtr;
			theDialogRect: Rect;
			eventReceived: Boolean;
			macEvent: EventRecord;

	begin
		new(theDialogTextP);

		theDialogText := @theDialogTextP;

		HLock(Handle(theDialogText));

		with theDialogTextP^ do
			begin
				numItems := 0;									{ To indicate one item.	}
				itemHandle := nil;								{ Just a placeholder.	}

				SetRect(itemRect, TextTopLeft, TextTopLeft, DialogRight, DialogBottom);
				itemTypeAndLength := textItemType + theTextLength;
				itemContent := theText;

{$PUSH}
{$R-}
				itemContent[0] := 'S';			{ Replace length byte with 'S'.		}
{$POP}

				itemContent[30] := CARRIAGE_RETURN;		{ Replace these spaces 	}
				itemContent[31] := CARRIAGE_RETURN;		{ with CRs				}
				itemContent[63] := CARRIAGE_RETURN;
			end;

		SetRect(theDialogRect, 0, 0, DialogRight, DialogBottom);

		with theDialogRect do			{ Center the dialog in the upper portion 	}
										{ of the screen.								}
			begin
				left := (screenBits.bounds.right - (right - left)) div 2;
				top := (screenBits.bounds.bottom - (bottom - top)) div 3;

				if top < GetMBarHeight + 7 then
					top := GetMBarHeight + 7;

				right := right + left;			{ left and top were both 0	}
				bottom := bottom + top;
			end;

		theDialog := NewDialog(nil, theDialogRect, nullStr, TRUE, dBoxProc, Pointer(-1), FALSE, 0, Handle(theDialogText));
		SysBeep(1);
		DrawDialog(theDialog);

{	Dismiss dialog on keyDown or mouseDown.	}

        repeat          { Altered by TCL Weaver 1.0 (4/27/90) }
            SystemTask;
            eventReceived := GetOSEvent(mDownMask + keyDownMask, macEvent);
		until eventReceived;

		DisposDialog(theDialog);
		gApplication.Exit;				{ Terminate program.	}
		ExitToShell;
	end;

end.