{****************************************************}
{ 		CRadioGroup.p}
{}
{		The RadioGroup Class}
{}
{		A group of radio buttons where only one button is selected at any }
{		one time. Clicking on one button in the group selects that }
{		button and deselects the previous choice. }
{}
{		SUPERCLASS = CBureaucrat}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CRadioGroup;

interface

	uses
		TCL;


implementation


{****************************************************}
{ IRadioGroup}
{}
{		Initialize a RadioGroup object}
{}
{****************************************************}

	procedure CRadioGroup.IRadioGroup (aSupervisor: CBureaucrat);

        var
            theButtons: CCluster;       { Altered by TCL Weaver 1.0 (5/9/90) }

    begin
        IBureaucrat(aSupervisor);       { Initialize superclass             }
        new(theButtons);                { Create cluster for storing the group's buttons    }
        itsButtons := theButtons;       { Altered by TCL Weaver 1.0 (5/9/90) }
        itsButtons.ICluster;
		station := nil;					{ No station is selected yet			}
	end;


{****************************************************}
{Free  (OVERRIDE)}
{}
{		Dispose of a RadioGroup object.}
{}
{****************************************************}

	procedure CRadioGroup.Free;
	begin
		itsButtons.Free;
		inherited Free;
	end;


{****************************************************}
{ AddButton}
{}
{		Add a RadioButton to the group}
{}
{****************************************************}

	procedure CRadioGroup.AddButton (theRadioButton: CRadioButton);
	begin
		itsButtons.Add(theRadioButton);
	end;


{****************************************************}
{ RemoveButton}
{}
{		Remove a RadioButton from the group}
{}
{****************************************************}

	procedure CRadioGroup.RemoveButton (theRadioButton: CRadioButton);
	begin
		itsButtons.Remove(theRadioButton);
	end;


{****************************************************}
{ AddButtonID}
{}
{		Add a RadioButton, specified by a CNTL ID number, to the group}
{}
{****************************************************}

	procedure CRadioGroup.AddButtonID (CNTLid: integer; anEnclosure: CView);
		var
			theRadioButton: CRadioButton;

	begin
		new(theRadioButton);
		theRadioButton.IRadioButton(CNTLid, anEnclosure, SELF);
		AddButton(theRadioButton);
	end;


{****************************************************}
{ ChangeStation}
{}
{		Change the current station}
{}
{****************************************************}

	procedure CRadioGroup.ChangeStation (theStation: CRadioButton);
	begin
		if station <> nil then
			station.SetValue(BUTTON_OFF);
		station := theStation;
		station.SetValue(BUTTON_ON);
	end;


{****************************************************}
{ GetStation}
{}
{		Return the current station}
{}
{****************************************************}

	function CRadioGroup.GetStation: CRadioButton;
	begin
		GetStation := station;
	end;


{****************************************************}
{ SetStationID}
{}
{		Set the current station, specified by an ID number}
{}
{****************************************************}

	function Test_StationID (theRadioButton: CRadioButton; theID: Ptr): Boolean;

{ Check for match of station ID numbers}

	begin
		Test_StationID := theRadioButton.GetIdNumber = IntPtr(theID)^;
	end;


	procedure CRadioGroup.SetStationID (aStationID: integer);
		var
			newStation: CRadioButton;

	begin
		if station <> nil then
			if station.GetIdNumber <> aStationID then
				station.SetValue(BUTTON_OFF);

		newStation := CRadioButton(itsButtons.FindItem1(Test_StationID, @aStationID));

		if newStation <> nil then
			begin
				station := newStation;
				station.SetValue(BUTTON_ON);
			end;
	end;


{****************************************************}
{ GetStationID}
{}
{		Return the current station's ID number}
{}
{****************************************************}

	function CRadioGroup.GetStationID: integer;
	begin
		if station = nil then
			GetStationID := NO_STATION
		else
			GetStationID := station.GetIdNumber
	end;

end.