{****************************************************}
{ 		CTearOffMenu.p}
{}
{		The TearOffMenu Class}
{}
{		Manages a menu and a Window which is displayed when the menu is torn off.}
{}
{		SUPERCLASS = CDirector}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CTearOffMenu;

interface

	uses
		TCL, MoreTCL;

implementation


{****************************************************}
{ ITearOffMenu}
{}
{		Initialize a TearOffMenu object}
{}
{****************************************************}

	procedure CTearOffMenu.ITearOffMenu (aSupervisor: CApplication; WINDid: integer);
        var
            theWindow: CWindow;         { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
		IDirector(aSupervisor);		{ Initialize superclass			}

        new(theWindow);             { Create a window to hold menu      }
                                        {   when it is torn off                 }
        itsWindow := theWindow;     { Altered by TCL Weaver 1.0 (5/9/90) }

		itsWindow.IWindow(WINDid, TRUE, gDesktop, SELF);
		itsWindow.SetActClick(TRUE);

		itsPane := nil;					{ Subclass must create the pane	}
		corner.h := 0;
		corner.v := 0;					{ Will be set when menu is torn	}
		SetRect(margins, 1, 1, 1, 1);	{ Space for a one pixel frame	}
	end;


{****************************************************}
{ Suspend (OVERRIDE)}
{}
{		Following the Human Interface Guidelines, tear-off menus are hidden}
{		when the application is suspended.}
{}
{****************************************************}

	procedure CTearOffMenu.Suspend;
	begin
		if active then
			begin
				itsWindow.HideSuspend;
				CFWDesktop(gDesktop).CalcTopFloat;
				active := TRUE;
			end;
	end;


{****************************************************}
{ Resume (OVERRIDE)}
{}
{		Resume by showing the tear-off window if it was visible before}
{		we were suspended.}
{}
{****************************************************}

	procedure CTearOffMenu.Resume;
	begin
		if active then
			begin
				itsWindow.ShowResume;
				CFWDesktop(gDesktop).CalcTopFloat;
			end;
	end;


{****************************************************}
{ CloseWind (OVERRIDE)}
{}
{		Close the floating window by moving it offscreen so it is no}
{		longer visible}
{}
{****************************************************}

	procedure CTearOffMenu.CloseWind (theWindow: CWindow);
	begin
		theWindow.MoveOffScreen;
	end;


{****************************************************}
{ TornOff}
{}
{		Menu has been torn off}
{}
{****************************************************}

	procedure CTearOffMenu.TornOff (aCorner: Point);
		var
			theTearChore: CTearChore;

	begin
		corner := aCorner;			{ Store location of tear off		}

		new(theTearChore);					{ Create and install an urgent		}
		theTearChore.ITearChore(SELF);	{   chore for moving the tear off	}
											{   menu to the new location			}

		gApplication.AssignUrgentChore(theTearChore);
	end;


{****************************************************}
{ MoveToCorner}
{}
{		Move the tear-off menu window to the location stored in the}
{		corner instance variable}
{}
{****************************************************}

	procedure CTearOffMenu.MoveToCorner;
	begin
		itsWindow.Move(corner.h, corner.v);
		itsWindow.Select;
	end;


{****************************************************}
{ GetMacWindow}
{}
{		Return a pointer to the Mac window record for the window associated}
{		with a TearOffMenu}
{}
{****************************************************}

	function CTearOffMenu.GetMacWindow: WindowPtr;
	begin
		GetMacWindow := itsWindow.GetMacPort;
	end;


{****************************************************}
{ SetMargins}
{}
{		Set the amount of margin between the window outline and the pane}
{		contained within the window. The margin includes the size of the}
{		frame and drag bar which form the structure region of the window}
{		used by a tear-off menu.}
{}
{****************************************************}

	procedure CTearOffMenu.SetMargins (aMargins: Rect);
	begin
		margins := aMargins;				{ Set instance variable			}
	end;


{****************************************************}
{ GetMargins}
{}
{		Get the amount of margin between the window outline and the pane}
{		contained within the window.}
{}
{****************************************************}

	procedure CTearOffMenu.GetMargins (var theMargins: Rect);
	begin
		theMargins := margins;				{ Return instance variable			}
	end;


end.